import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { ApplicationRef, Component, ComponentFactoryResolver, ElementRef, HostBinding, Injectable, Injector, Input, NgModule, Renderer } from '@angular/core';
import { CommonModule } from '@angular/common';
import { animate, style, transition, trigger } from '@angular/animations';

var DomService = (function () {
    /**
     * @param {?} componentFactoryResolver
     * @param {?} appRef
     * @param {?} injector
     */
    function DomService(componentFactoryResolver, appRef, injector) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.appRef = appRef;
        this.injector = injector;
    }
    /**
     * @param {?} component
     * @return {?}
     */
    DomService.prototype.createComponentRef = function (component) {
        var /** @type {?} */ componentRef = this.componentFactoryResolver
            .resolveComponentFactory(component)
            .create(this.injector);
        this.appRef.attachView(componentRef.hostView);
        return componentRef;
    };
    /**
     * @param {?} componentRef
     * @return {?}
     */
    DomService.prototype.getDomElementFromComponentRef = function (componentRef) {
        return (((componentRef.hostView))
            .rootNodes[0]);
    };
    /**
     * @param {?} child
     * @param {?=} parent
     * @return {?}
     */
    DomService.prototype.addChild = function (child, parent) {
        if (parent === void 0) { parent = document.body; }
        parent.appendChild(child);
    };
    /**
     * @param {?} componentRef
     * @param {?} delay
     * @return {?}
     */
    DomService.prototype.destroyRef = function (componentRef, delay) {
        var _this = this;
        setTimeout(function () {
            _this.appRef.detachView(componentRef.hostView);
            componentRef.destroy();
        }, delay);
    };
    /**
     * @param {?} styles
     * @param {?} componentRef
     * @return {?}
     */
    DomService.prototype.setDynamicStyles = function (styles, componentRef) {
        Object.keys(styles).forEach(function (cssRule) {
            var /** @type {?} */ cssValue = styles[cssRule];
            componentRef.instance.renderer.setElementStyle(componentRef.instance.elementRef.nativeElement, cssRule, cssValue);
        });
    };
    return DomService;
}());
DomService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
DomService.ctorParameters = function () { return [
    { type: ComponentFactoryResolver, },
    { type: ApplicationRef, },
    { type: Injector, },
]; };

var ToastContainerComponent = (function () {
    /**
     * @param {?} elementRef
     * @param {?} renderer
     */
    function ToastContainerComponent(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
    }
    /**
     * @return {?}
     */
    ToastContainerComponent.prototype.ngOnInit = function () { };
    return ToastContainerComponent;
}());
ToastContainerComponent.decorators = [
    { type: Component, args: [{
                selector: 'notyf-toast-container',
                template: "",
                styles: [":host {  display: block; position: absolute; bottom: 0; right: 0; }"]
            },] },
];
/**
 * @nocollapse
 */
ToastContainerComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer, },
]; };

var flyInOut = trigger('flyInOut', [
    transition('void => *', [
        style({
            transform: 'translateY(0) scaleX(0.75)',
            marginBottom: '-50px',
            opacity: 0
        }),
        animate('0.4s cubic-bezier(0.23, 1, 0.32, 1)')
    ]),
    transition('* => void', [
        animate('0.4s cubic-bezier(0.23, 1, 0.32, 1)', style({
            transform: 'translateX(10px)',
            opacity: 0
        }))
    ])
]);
var ToastComponent = (function () {
    /**
     * @param {?} elementRef
     * @param {?} renderer
     */
    function ToastComponent(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
    }
    /**
     * @return {?}
     */
    ToastComponent.prototype.ngOnInit = function () {
        this.success = this.type === ToastType.Success;
        this.error = this.type === ToastType.Error;
    };
    return ToastComponent;
}());
ToastComponent.decorators = [
    { type: Component, args: [{
                selector: 'notyf-toast',
                template: "{{message}}",
                animations: [flyInOut],
                host: {
                    '[@flyInOut]': 'true'
                },
                styles: [":host { background: #323232; display: block; color: white; padding: 1rem 1rem; margin: 1rem 1rem; text-align: left; border-radius: 2px; box-shadow: 0 2px 10px rgba(0, 0, 0, 0.34); box-sizing: border-box; transform-origin: right; } :host(.notyf--error) { background: crimson; } :host(.notyf--success) { background: greenyellow; }"]
            },] },
];
/**
 * @nocollapse
 */
ToastComponent.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: Renderer, },
]; };
ToastComponent.propDecorators = {
    'message': [{ type: Input },],
    'type': [{ type: Input },],
    'success': [{ type: HostBinding, args: ['class.notyf--success',] },],
    'error': [{ type: HostBinding, args: ['class.notyf--error',] },],
};
var ToastType = {};
ToastType.Success = 0;
ToastType.Error = 1;
ToastType[ToastType.Success] = "Success";
ToastType[ToastType.Error] = "Error";

var NotyfService = (function () {
    /**
     * @param {?} domService
     */
    function NotyfService(domService) {
        this.domService = domService;
        this._toastDelay = 2000;
        this._toastStyle = {};
        this._toastContainerStyle = {};
        this.toastContainerRef = this.domService.createComponentRef(ToastContainerComponent);
        this.toastContainerElement = this.domService.getDomElementFromComponentRef(this.toastContainerRef);
        this.domService.addChild(this.toastContainerElement);
    }
    /**
     * @param {?} message
     * @return {?}
     */
    NotyfService.prototype.success = function (message) { this.addToast(message, ToastType.Success); };
    /**
     * @param {?} message
     * @return {?}
     */
    NotyfService.prototype.error = function (message) { this.addToast(message, ToastType.Error); };
    /**
     * @param {?} message
     * @param {?} type
     * @return {?}
     */
    NotyfService.prototype.addToast = function (message, type) {
        var /** @type {?} */ toastRef = this.domService.createComponentRef(ToastComponent);
        this.domService.setDynamicStyles(this.toastStyle, toastRef);
        toastRef.instance.message = message;
        toastRef.instance.type = type;
        var /** @type {?} */ toastElement = this.domService.getDomElementFromComponentRef(toastRef);
        this.domService.addChild(toastElement, this.toastContainerElement);
        this.domService.destroyRef(toastRef, this.toastDelay);
    };
    Object.defineProperty(NotyfService.prototype, "toastDelay", {
        /**
         * @return {?}
         */
        get: function () {
            return this._toastDelay;
        },
        /**
         * @param {?} toastDelay
         * @return {?}
         */
        set: function (toastDelay) {
            this._toastDelay = toastDelay > 0 ? toastDelay : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NotyfService.prototype, "toastStyle", {
        /**
         * @return {?}
         */
        get: function () {
            return this._toastStyle;
        },
        /**
         * @param {?} toastStyle
         * @return {?}
         */
        set: function (toastStyle) {
            this._toastStyle = toastStyle;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NotyfService.prototype, "toastContainerStyle", {
        /**
         * @return {?}
         */
        get: function () {
            return this._toastContainerStyle;
        },
        /**
         * @param {?} toastContainerStyle
         * @return {?}
         */
        set: function (toastContainerStyle) {
            this._toastContainerStyle = toastContainerStyle;
            this.domService.setDynamicStyles(this.toastContainerStyle, this.toastContainerRef);
        },
        enumerable: true,
        configurable: true
    });
    return NotyfService;
}());
NotyfService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
NotyfService.ctorParameters = function () { return [
    { type: DomService, },
]; };

var NotyfModule = (function () {
    function NotyfModule() {
    }
    return NotyfModule;
}());
NotyfModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, BrowserAnimationsModule],
                declarations: [ToastComponent, ToastContainerComponent],
                exports: [],
                entryComponents: [ToastComponent, ToastContainerComponent],
                providers: [NotyfService, DomService]
            },] },
];
/**
 * @nocollapse
 */
NotyfModule.ctorParameters = function () { return []; };

export { NotyfService, NotyfModule };
